@@ -291,12 +291,8 @@ public class UndertowOutputStream extends OutputStream implements BufferWritable
         buffer.flip();
 
         while (buffer.hasRemaining()) {
-            if(writeFinal) {
-                channel.writeFinal(buffer);
-            } else {
-                channel.write(buffer);
-            }
-            if(buffer.hasRemaining()) {
+            int result = writeFinal ? channel.writeFinal(buffer) : channel.write(buffer);
+            if (result == 0) {
                 channel.awaitWritable();
             }
         }
