@@ -18,13 +18,30 @@
 
 package io.undertow.protocols.ssl;
 
+import java.io.IOException;
+import java.io.InterruptedIOException;
+import java.nio.ByteBuffer;
+import java.nio.channels.ClosedChannelException;
+import java.nio.channels.FileChannel;
+import java.util.ArrayList;
+import java.util.List;
+import java.util.concurrent.Executor;
+import java.util.concurrent.RejectedExecutionException;
+import java.util.concurrent.TimeUnit;
+
+import javax.net.ssl.SSLEngine;
+import javax.net.ssl.SSLEngineResult;
+import javax.net.ssl.SSLException;
+import javax.net.ssl.SSLSession;
+
 import io.undertow.UndertowLogger;
+import io.undertow.connector.ByteBufferPool;
+import io.undertow.connector.PooledByteBuffer;
+import io.undertow.server.DefaultByteBufferPool;
 import org.xnio.Buffers;
 import org.xnio.ChannelListener;
 import org.xnio.ChannelListeners;
 import org.xnio.IoUtils;
-import io.undertow.connector.ByteBufferPool;
-import io.undertow.connector.PooledByteBuffer;
 import org.xnio.StreamConnection;
 import org.xnio.XnioIoThread;
 import org.xnio.XnioWorker;
@@ -40,25 +57,15 @@ import org.xnio.conduits.StreamSinkConduit;
 import org.xnio.conduits.StreamSourceConduit;
 import org.xnio.conduits.WriteReadyHandler;
 
-import javax.net.ssl.SSLEngine;
-import javax.net.ssl.SSLEngineResult;
-import javax.net.ssl.SSLException;
-import javax.net.ssl.SSLSession;
-import java.io.IOException;
-import java.io.InterruptedIOException;
-import java.nio.ByteBuffer;
-import java.nio.channels.ClosedChannelException;
-import java.nio.channels.FileChannel;
-import java.util.ArrayList;
-import java.util.List;
-import java.util.concurrent.TimeUnit;
-
 import static org.xnio.Bits.allAreClear;
 import static org.xnio.Bits.allAreSet;
 import static org.xnio.Bits.anyAreSet;
 
 /**
+ * Conduit for SSL connections.
+ *
  * @author Stuart Douglas
+ * @author Flavia Rainone
  */
 public class SslConduit implements StreamSourceConduit, StreamSinkConduit {
 
@@ -117,9 +124,17 @@ public class SslConduit implements StreamSourceConduit, StreamSinkConduit {
     private static final int FLAG_READ_CLOSED = 1 << 14;
     public static final ByteBuffer EMPTY_BUFFER = ByteBuffer.allocate(0);
 
+    /**
+     * Buffer pool created and used only when large fragments handling is
+     * enabled in the underlying SSL Engine. When this happens, we need
+     * a specific buffer with expanded capacity.
+     */
+    private static volatile ByteBufferPool expandedBufferPool;
+
 
     private final UndertowSslConnection connection;
     private final StreamConnection delegate;
+    private final Executor delegatedTaskExecutor;
     private SSLEngine engine;
     private final StreamSinkConduit sink;
     private final StreamSourceConduit source;
@@ -184,13 +199,14 @@ public class SslConduit implements StreamSourceConduit, StreamSinkConduit {
         }
     };
 
-    SslConduit(UndertowSslConnection connection, StreamConnection delegate, SSLEngine engine, ByteBufferPool bufferPool, Runnable handshakeCallback) {
+    SslConduit(UndertowSslConnection connection, StreamConnection delegate, SSLEngine engine, Executor delegatedTaskExecutor, ByteBufferPool bufferPool, Runnable handshakeCallback) {
         this.connection = connection;
         this.delegate = delegate;
         this.handshakeCallback = handshakeCallback;
         this.sink = delegate.getSinkChannel().getConduit();
         this.source = delegate.getSourceChannel().getConduit();
         this.engine = engine;
+        this.delegatedTaskExecutor = delegatedTaskExecutor;
         this.bufferPool = bufferPool;
         delegate.getSourceChannel().getConduit().setReadReadyHandler(readReadyHandler = new SslReadReadyHandler(null));
         delegate.getSinkChannel().getConduit().setWriteReadyHandler(writeReadyHandler = new SslWriteReadyHandler(null));
@@ -295,6 +311,7 @@ public class SslConduit implements StreamSourceConduit, StreamSinkConduit {
                     wait();
                     return;
                 } catch (InterruptedException e) {
+                    Thread.currentThread().interrupt();
                     throw new InterruptedIOException();
                 }
             }
@@ -320,6 +337,7 @@ public class SslConduit implements StreamSourceConduit, StreamSinkConduit {
                     wait(timeUnit.toMillis(time));
                     return;
                 } catch (InterruptedException e) {
+                    Thread.currentThread().interrupt();
                     throw new InterruptedIOException();
                 }
             }
@@ -449,6 +467,7 @@ public class SslConduit implements StreamSourceConduit, StreamSinkConduit {
                         this.wait();
                         return;
                     } catch (InterruptedException e) {
+                        Thread.currentThread().interrupt();
                         throw new InterruptedIOException();
                     }
                 }
@@ -473,6 +492,7 @@ public class SslConduit implements StreamSourceConduit, StreamSinkConduit {
                         this.wait(timeUnit.toMillis(time));
                         return;
                     } catch (InterruptedException e) {
+                        Thread.currentThread().interrupt();
                         throw new InterruptedIOException();
                     }
                 }
@@ -580,6 +600,10 @@ public class SslConduit implements StreamSourceConduit, StreamSinkConduit {
         return delegate.getWorker();
     }
 
+    private Executor getDelegatedTaskExecutor() {
+        return delegatedTaskExecutor == null ? getWorker() : delegatedTaskExecutor;
+    }
+
     void notifyWriteClosed() {
         if(anyAreSet(state, FLAG_WRITE_CLOSED)) {
             return;
@@ -649,6 +673,27 @@ public class SslConduit implements StreamSourceConduit, StreamSinkConduit {
         doWrap(null, 0, 0);
     }
 
+    /**
+     * Perform unwraps while it is needed and there is data in src buffer.
+     *
+     * @param src The src wrapped data to unwrap.
+     * @param dsts The destinations buffers to unwrap the data to
+     * @param off Offset buffer
+     * @param len len buffer
+     * @param bytesProduced Accumulative or boolean for the bytes produced
+     * @return The last engine result
+     * @throws IOException Some error unwrapping
+     */
+    private SSLEngineResult engineUnwrap(ByteBuffer src, ByteBuffer[] dsts, int off,
+            int len, AccumulativeOrBoolean bytesProduced) throws IOException {
+        SSLEngineResult result;
+        do {
+            result = engine.unwrap(src, dsts, off, len);
+            bytesProduced.add(result.bytesProduced() > 0);
+        } while (result.getHandshakeStatus() == SSLEngineResult.HandshakeStatus.NEED_UNWRAP
+                && result.getStatus() == SSLEngineResult.Status.OK && src.hasRemaining());
+        return result;
+    }
 
     /**
      * Unwrap channel data into the user buffers. If no user buffer is supplied (e.g. during handshaking) then the
@@ -660,7 +705,7 @@ public class SslConduit implements StreamSourceConduit, StreamSinkConduit {
      * @return true if the unwrap operation made progress, false otherwise
      * @throws SSLException
      */
-    private long doUnwrap(ByteBuffer[] userBuffers, int off, int len) throws IOException {
+    private synchronized long doUnwrap(ByteBuffer[] userBuffers, int off, int len) throws IOException {
         if(anyAreSet(state, FLAG_CLOSED)) {
             throw new ClosedChannelException();
         }
@@ -673,7 +718,7 @@ public class SslConduit implements StreamSourceConduit, StreamSinkConduit {
                 return 0;
             }
         }
-        boolean bytesProduced = false;
+        AccumulativeOrBoolean bytesProduced = new AccumulativeOrBoolean();
         PooledByteBuffer unwrappedData = this.unwrappedData;
         //copy any exiting data
         if(unwrappedData != null) {
@@ -735,7 +780,7 @@ public class SslConduit implements StreamSourceConduit, StreamSinkConduit {
             boolean unwrapBufferUsed = false;
             try {
                 if (userBuffers != null) {
-                    result = engine.unwrap(this.dataToUnwrap.getBuffer(), userBuffers, off, len);
+                    result = engineUnwrap(this.dataToUnwrap.getBuffer(), userBuffers, off, len, bytesProduced);
                     if (result.getStatus() == SSLEngineResult.Status.BUFFER_OVERFLOW) {
                         //not enough space in the user buffers
                         //we use our own
@@ -743,10 +788,9 @@ public class SslConduit implements StreamSourceConduit, StreamSinkConduit {
                         ByteBuffer[] d = new ByteBuffer[len + 1];
                         System.arraycopy(userBuffers, off, d, 0, len);
                         d[len] = unwrappedData.getBuffer();
-                        result = engine.unwrap(this.dataToUnwrap.getBuffer(), d);
+                        result = engineUnwrap(this.dataToUnwrap.getBuffer(), d, 0, d.length, bytesProduced);
                         unwrapBufferUsed = true;
                     }
-                    bytesProduced = result.bytesProduced() > 0;
                 } else {
                     unwrapBufferUsed = true;
                     if (unwrappedData == null) {
@@ -754,8 +798,7 @@ public class SslConduit implements StreamSourceConduit, StreamSinkConduit {
                     } else {
                         unwrappedData.getBuffer().compact();
                     }
-                    result = engine.unwrap(this.dataToUnwrap.getBuffer(), unwrappedData.getBuffer());
-                    bytesProduced = result.bytesProduced() > 0;
+                    result = engineUnwrap(this.dataToUnwrap.getBuffer(), new ByteBuffer[] {unwrappedData.getBuffer()}, 0, 1, bytesProduced);
                 }
             } finally {
                 if (unwrapBufferUsed) {
@@ -834,7 +877,7 @@ public class SslConduit implements StreamSourceConduit, StreamSinkConduit {
         } finally {
             boolean requiresListenerInvocation = false; //if there is data in the buffer and reads are resumed we should re-run the listener
             //we always need to re-invoke if bytes have been produced, as the engine may have buffered some data
-            if (bytesProduced || (unwrappedData != null && unwrappedData.isOpen() && unwrappedData.getBuffer().hasRemaining())) {
+            if (bytesProduced.get() || (unwrappedData != null && unwrappedData.isOpen() && unwrappedData.getBuffer().hasRemaining())) {
                 requiresListenerInvocation = true;
             }
             if (dataToUnwrap != null) {
@@ -871,7 +914,7 @@ public class SslConduit implements StreamSourceConduit, StreamSinkConduit {
      * @return The amount of data consumed
      * @throws IOException
      */
-    private long doWrap(ByteBuffer[] userBuffers, int off, int len) throws IOException {
+    private synchronized long doWrap(ByteBuffer[] userBuffers, int off, int len) throws IOException {
         if(anyAreSet(state, FLAG_CLOSED)) {
             throw new ClosedChannelException();
         }
@@ -880,7 +923,7 @@ public class SslConduit implements StreamSourceConduit, StreamSinkConduit {
         }
         if(anyAreSet(state, FLAG_WRITE_REQUIRES_READ)) {
             doUnwrap(null, 0, 0);
-            if(allAreClear(state, FLAG_READ_REQUIRES_WRITE)) { //unless a wrap is immediatly required we just return
+            if(allAreClear(state, FLAG_READ_REQUIRES_WRITE)) { //unless a wrap is immediately required we just return
                 return 0;
             }
         }
@@ -894,21 +937,27 @@ public class SslConduit implements StreamSourceConduit, StreamSinkConduit {
             wrappedData = bufferPool.allocate();
         }
         try {
-            SSLEngineResult result = null;
-            while (result == null || (result.getHandshakeStatus() == SSLEngineResult.HandshakeStatus.NEED_WRAP && result.getStatus() != SSLEngineResult.Status.BUFFER_OVERFLOW)) {
-                if (userBuffers == null) {
-                    result = engine.wrap(EMPTY_BUFFER, wrappedData.getBuffer());
-                } else {
-                    result = engine.wrap(userBuffers, off, len, wrappedData.getBuffer());
-                }
-            }
-            wrappedData.getBuffer().flip();
+            SSLEngineResult result = wrapAndFlip(userBuffers, off, len);
 
             if (result.getStatus() == SSLEngineResult.Status.BUFFER_UNDERFLOW) {
-                throw new IOException("underflow"); //todo: can this happen?
+                throw new IOException("underflow"); // unexpected result
             } else if (result.getStatus() == SSLEngineResult.Status.BUFFER_OVERFLOW) {
-                if (!wrappedData.getBuffer().hasRemaining()) { //if an earlier wrap suceeded we ignore this
-                    throw new IOException("overflow"); //todo: handle properly
+                //if an earlier wrap succeeded we ignore this
+                if (!wrappedData.getBuffer().hasRemaining()) {
+                    if (wrappedData.getBuffer().capacity() < engine.getSession().getPacketBufferSize()) {
+                        wrappedData.close();
+                        final int bufferSize = engine.getSession().getPacketBufferSize();
+                        UndertowLogger.REQUEST_IO_LOGGER.tracev(
+                                "Expanded buffer enabled due to overflow with empty buffer, buffer size is %s", bufferSize);
+                        if (expandedBufferPool == null || expandedBufferPool.getBufferSize() < bufferSize)
+                            expandedBufferPool = new DefaultByteBufferPool(false, bufferSize, -1, 12);
+                        wrappedData = expandedBufferPool.allocate();
+                        result = wrapAndFlip(userBuffers, off, len);
+                        if (result.getStatus() == SSLEngineResult.Status.BUFFER_OVERFLOW &&
+                                !wrappedData.getBuffer().hasRemaining())
+                            throw new IOException("overflow"); // unexpected result
+                    }
+                    else throw new IOException("overflow"); // unexpected result
                 }
             }
             //attempt to write it out, if we fail we just return
@@ -948,6 +997,19 @@ public class SslConduit implements StreamSourceConduit, StreamSinkConduit {
         }
     }
 
+    private SSLEngineResult wrapAndFlip(ByteBuffer[] userBuffers, int off, int len) throws IOException {
+        SSLEngineResult result = null;
+        while (result == null || (result.getHandshakeStatus() == SSLEngineResult.HandshakeStatus.NEED_WRAP && result.getStatus() != SSLEngineResult.Status.BUFFER_OVERFLOW)) {
+            if (userBuffers == null) {
+                result = engine.wrap(EMPTY_BUFFER, wrappedData.getBuffer());
+            } else {
+                result = engine.wrap(userBuffers, off, len, wrappedData.getBuffer());
+            }
+        }
+        wrappedData.getBuffer().flip();
+        return result;
+    }
+
     private boolean handleHandshakeResult(SSLEngineResult result) throws IOException {
         switch (result.getHandshakeStatus()) {
             case NEED_TASK: {
@@ -1016,42 +1078,44 @@ public class SslConduit implements StreamSourceConduit, StreamSinkConduit {
         if(anyAreSet(state, FLAG_CLOSED)) {
             return;
         }
-        state |= FLAG_CLOSED | FLAG_DELEGATE_SINK_SHUTDOWN | FLAG_DELEGATE_SOURCE_SHUTDOWN | FLAG_WRITE_SHUTDOWN | FLAG_READ_SHUTDOWN;
-        notifyReadClosed();
-        notifyWriteClosed();
-        if(dataToUnwrap != null) {
-            dataToUnwrap.close();
-            dataToUnwrap = null;
-        }
-        if(unwrappedData != null) {
-            unwrappedData.close();
-            unwrappedData = null;
-        }
-        if(wrappedData != null) {
-            wrappedData.close();
-            wrappedData = null;
-        }
-        if(allAreClear(state, FLAG_ENGINE_OUTBOUND_SHUTDOWN)) {
-            engine.closeOutbound();
-        }
-        if(allAreClear(state, FLAG_ENGINE_INBOUND_SHUTDOWN)) {
-            try {
-                engine.closeInbound();
-            } catch (SSLException e) {
-                UndertowLogger.REQUEST_LOGGER.ioException(e);
-            } catch (Throwable t) {
-                UndertowLogger.REQUEST_LOGGER.handleUnexpectedFailure(t);
+        synchronized (this) {
+            state |= FLAG_CLOSED | FLAG_DELEGATE_SINK_SHUTDOWN | FLAG_DELEGATE_SOURCE_SHUTDOWN | FLAG_WRITE_SHUTDOWN | FLAG_READ_SHUTDOWN;
+            notifyReadClosed();
+            notifyWriteClosed();
+            if (dataToUnwrap != null) {
+                dataToUnwrap.close();
+                dataToUnwrap = null;
+            }
+            if (unwrappedData != null) {
+                unwrappedData.close();
+                unwrappedData = null;
+            }
+            if (wrappedData != null) {
+                wrappedData.close();
+                wrappedData = null;
+            }
+            if (allAreClear(state, FLAG_ENGINE_OUTBOUND_SHUTDOWN)) {
+                engine.closeOutbound();
+            }
+            if (allAreClear(state, FLAG_ENGINE_INBOUND_SHUTDOWN)) {
+                try {
+                    engine.closeInbound();
+                } catch (SSLException e) {
+                    UndertowLogger.REQUEST_LOGGER.ioException(e);
+                } catch (Throwable t) {
+                    UndertowLogger.REQUEST_LOGGER.handleUnexpectedFailure(t);
+                }
             }
         }
         IoUtils.safeClose(delegate);
     }
 
     /**
-     * Execute all the tasks in the worker
+     * Execute all the delegated tasks on an executor which allows blocking, the worker executor by default.
      *
      * Once they are complete we notify any waiting threads and wakeup reads/writes as appropriate
      */
-    private void runTasks() {
+    private void runTasks() throws IOException {
         //don't run anything in the IO thread till the tasks are done
         delegate.getSinkChannel().suspendWrites();
         delegate.getSourceChannel().suspendReads();
@@ -1065,7 +1129,7 @@ public class SslConduit implements StreamSourceConduit, StreamSinkConduit {
         synchronized (this) {
             outstandingTasks += tasks.size();
             for (final Runnable task : tasks) {
-                getWorker().execute(new Runnable() {
+                Runnable wrappedTask = new Runnable() {
                     @Override
                     public void run() {
                         try {
@@ -1083,7 +1147,7 @@ public class SslConduit implements StreamSourceConduit, StreamSinkConduit {
                                                 try {
                                                     doHandshake();
                                                 } catch (IOException | RuntimeException | Error e) {
-                                                    UndertowLogger.REQUEST_LOGGER.error("Closing SSLConduit after exception on handshake", e);
+                                                    UndertowLogger.REQUEST_LOGGER.debug("Closing SSLConduit after exception on handshake", e);
                                                     IoUtils.safeClose(connection);
                                                 }
                                                 if (anyAreSet(state, FLAG_READS_RESUMED)) {
@@ -1100,13 +1164,46 @@ public class SslConduit implements StreamSourceConduit, StreamSinkConduit {
                                 }
                             }
                         }
-
                     }
-                });
+                };
+                try {
+                    getDelegatedTaskExecutor().execute(wrappedTask);
+                } catch (RejectedExecutionException e) {
+                    UndertowLogger.REQUEST_IO_LOGGER.sslEngineDelegatedTaskRejected(e);
+                    IoUtils.safeClose(connection);
+                    throw DelegatedTaskRejectedClosedChannelException.INSTANCE;
+                }
             }
         }
     }
 
+    /**
+     * A specialized {@link ClosedChannelException} which does not provide a stack trace. Tasks may be rejected
+     * when the server is overloaded, so it's important not to create more work than necessary.
+     */
+    private static final class DelegatedTaskRejectedClosedChannelException extends ClosedChannelException {
+
+        private static final DelegatedTaskRejectedClosedChannelException INSTANCE =
+                new DelegatedTaskRejectedClosedChannelException();
+
+        @Override
+        public Throwable fillInStackTrace() {
+            // Avoid the most expensive part of exception creation.
+            return this;
+        }
+
+        // Ignore mutations
+        @Override
+        public Throwable initCause(Throwable ignored) {
+            return this;
+        }
+
+        @Override
+        public void setStackTrace(StackTraceElement[] ignored) {
+            // no-op
+        }
+    }
+
     public SSLEngine getSSLEngine() {
         return engine;
     }
@@ -1293,4 +1390,16 @@ public class SslConduit implements StreamSourceConduit, StreamSinkConduit {
                 ", unwrappedData=" + unwrappedData +
                 '}';
     }
+
+    private static class AccumulativeOrBoolean {
+        private boolean value = false;
+
+        public void add(boolean value) {
+            this.value = this.value || value;
+        }
+
+        public boolean get() {
+            return this.value;
+        }
+    }
 }
