@@ -26,7 +26,7 @@ import java.util.Date;
  * @see io.undertow.server.Connectors
  * @author Stuart Douglas
  */
-public interface Cookie {
+public interface Cookie extends Comparable {
 
     String getName();
 
@@ -82,7 +82,34 @@ public interface Cookie {
         return null;
     }
 
-    default Cookie setSameSiteMode(final String sameSiteMode) {
+    default Cookie setSameSiteMode(final String mode) {
         throw new UnsupportedOperationException("Not implemented");
     }
+
+    @Override
+    default int compareTo(final Object other) {
+        final Cookie o = (Cookie) other;
+        int retVal = 0;
+
+        // compare names
+        if (getName() == null && o.getName() != null) return -1;
+        if (getName() != null && o.getName() == null) return 1;
+        retVal = (getName() == null && o.getName() == null) ? 0 : getName().compareTo(o.getName());
+        if (retVal != 0) return retVal;
+
+        // compare paths
+        if (getPath() == null && o.getPath() != null) return -1;
+        if (getPath() != null && o.getPath() == null) return 1;
+        retVal = (getPath() == null && o.getPath() == null) ? 0 : getPath().compareTo(o.getPath());
+        if (retVal != 0) return retVal;
+
+        // compare domains
+        if (getDomain() == null && o.getDomain() != null) return -1;
+        if (getDomain() != null && o.getDomain() == null) return 1;
+        retVal = (getDomain() == null && o.getDomain() == null) ? 0 : getDomain().compareTo(o.getDomain());
+        if (retVal != 0) return retVal;
+
+        return 0; // equal
+    }
+
 }
