@@ -261,6 +261,62 @@ public class CookiesTestCase {
         Assert.assertEquals("FEDEX", cookie.getValue());
     }
 
+    @Test
+    public void testCookieContainsColonInJvmRoute() {
+        // "<hostcontroller-name>:<server-name>" (e.g. master:node1) is added as jvmRoute (instance-id) by default in WildFly domain mode.
+        // ":" is http separator, so it's not allowed in V0 cookie value.
+        // However, we need to allow it exceptionally by default. Because, when Undertow runs as a proxy server (like mod_cluster),
+        // we need to handle jvmRoute containing ":" in the request cookie value correctly to maintain the sticky session.
+
+        Map<String, Cookie> cookies = Cookies.parseRequestCookies(3, false, Arrays.asList("JSESSIONID=WCGWBPJ8DUmv0fvREqVQZb8E6bzW92iHnzysV_q_.master:node1; CUSTOMER=WILE_E COYOTE; SHIPPING=FEDEX" ), true, false);
+        Assert.assertEquals(3, cookies.size());
+        Cookie cookie = cookies.get("CUSTOMER");
+        Assert.assertNotNull(cookie);
+        Assert.assertEquals("WILE_E", cookie.getValue());
+        cookie = cookies.get("SHIPPING");
+        Assert.assertNotNull(cookie);
+        Assert.assertEquals("FEDEX", cookie.getValue());
+        cookie = cookies.get("JSESSIONID");
+        Assert.assertNotNull(cookie);
+        Assert.assertEquals("WCGWBPJ8DUmv0fvREqVQZb8E6bzW92iHnzysV_q_.master:node1", cookie.getValue());
+
+        cookies = Cookies.parseRequestCookies(3, false, Arrays.asList("JSESSIONID=WCGWBPJ8DUmv0fvREqVQZb8E6bzW92iHnzysV_q_.master:node1; CUSTOMER=WILE_E COYOTE; SHIPPING=FEDEX" ), true, true);
+        Assert.assertEquals(3, cookies.size());
+        cookie = cookies.get("CUSTOMER");
+        Assert.assertNotNull(cookie);
+        Assert.assertEquals("WILE_E COYOTE", cookie.getValue());
+        cookie = cookies.get("SHIPPING");
+        Assert.assertNotNull(cookie);
+        Assert.assertEquals("FEDEX", cookie.getValue());
+        cookie = cookies.get("JSESSIONID");
+        Assert.assertNotNull(cookie);
+        Assert.assertEquals("WCGWBPJ8DUmv0fvREqVQZb8E6bzW92iHnzysV_q_.master:node1", cookie.getValue());
+
+        cookies = Cookies.parseRequestCookies(3, false, Arrays.asList("JSESSIONID=WCGWBPJ8DUmv0fvREqVQZb8E6bzW92iHnzysV_q_.master:node1; CUSTOMER=WILE_E_COYOTE\"; SHIPPING=FEDEX" ), true, false);
+        Assert.assertEquals(3, cookies.size());
+        cookie = cookies.get("CUSTOMER");
+        Assert.assertNotNull(cookie);
+        Assert.assertEquals("WILE_E_COYOTE", cookie.getValue());
+        cookie = cookies.get("SHIPPING");
+        Assert.assertNotNull(cookie);
+        Assert.assertEquals("FEDEX", cookie.getValue());
+        cookie = cookies.get("JSESSIONID");
+        Assert.assertNotNull(cookie);
+        Assert.assertEquals("WCGWBPJ8DUmv0fvREqVQZb8E6bzW92iHnzysV_q_.master:node1", cookie.getValue());
+
+        cookies = Cookies.parseRequestCookies(3, false, Arrays.asList("JSESSIONID=WCGWBPJ8DUmv0fvREqVQZb8E6bzW92iHnzysV_q_.master:node1; CUSTOMER=WILE_E_COYOTE\"; SHIPPING=FEDEX" ), true, true);
+        Assert.assertEquals(3, cookies.size());
+        cookie = cookies.get("CUSTOMER");
+        Assert.assertNotNull(cookie);
+        Assert.assertEquals("WILE_E_COYOTE\"", cookie.getValue());
+        cookie = cookies.get("SHIPPING");
+        Assert.assertNotNull(cookie);
+        Assert.assertEquals("FEDEX", cookie.getValue());
+        cookie = cookies.get("JSESSIONID");
+        Assert.assertNotNull(cookie);
+        Assert.assertEquals("WCGWBPJ8DUmv0fvREqVQZb8E6bzW92iHnzysV_q_.master:node1", cookie.getValue());
+    }
+
     @Test
     public void testQuotedEscapedStringInRequestCookie() {
         Map<String, Cookie> cookies = Cookies.parseRequestCookies(3, false, Arrays.asList(
@@ -362,31 +418,35 @@ public class CookiesTestCase {
 
     @Test
     public void testSameSiteCookie() {
-        Cookie cookie = Cookies.parseSetCookieHeader("CUSTOMER=WILE_E_COYOTE; path=/; SameSite");
+        Cookie cookie = Cookies.parseSetCookieHeader("CUSTOMER=WILE_E_COYOTE; path=/");
         Assert.assertEquals("CUSTOMER", cookie.getName());
         Assert.assertEquals("WILE_E_COYOTE", cookie.getValue());
         Assert.assertEquals("/", cookie.getPath());
-        Assert.assertTrue(cookie.isSameSite());
         Assert.assertNull(cookie.getSameSiteMode());
 
+        cookie = Cookies.parseSetCookieHeader("CUSTOMER=WILE_E_COYOTE; path=/; SameSite=None");
+        Assert.assertEquals("CUSTOMER", cookie.getName());
+        Assert.assertEquals("WILE_E_COYOTE", cookie.getValue());
+        Assert.assertEquals("/", cookie.getPath());
+        Assert.assertEquals("None", cookie.getSameSiteMode());
+
         cookie = Cookies.parseSetCookieHeader("SHIPPING=FEDEX; path=/foo; SameSite=Strict");
         Assert.assertEquals("SHIPPING", cookie.getName());
         Assert.assertEquals("FEDEX", cookie.getValue());
         Assert.assertEquals("/foo", cookie.getPath());
-        Assert.assertTrue(cookie.isSameSite());
         Assert.assertEquals("Strict", cookie.getSameSiteMode());
 
         cookie = Cookies.parseSetCookieHeader("SHIPPING=FEDEX; path=/acme; SameSite=Lax");
         Assert.assertEquals("SHIPPING", cookie.getName());
         Assert.assertEquals("FEDEX", cookie.getValue());
         Assert.assertEquals("/acme", cookie.getPath());
-        Assert.assertTrue(cookie.isSameSite());
         Assert.assertEquals("Lax", cookie.getSameSiteMode());
-    }
 
-    @Test(expected = IllegalArgumentException.class)
-    public void testInvalidSameSiteCookie() {
-        Cookie cookie = Cookies.parseSetCookieHeader("CUSTOMER=WILE_E_COYOTE; path=/; SameSite=test");
+        cookie = Cookies.parseSetCookieHeader("CUSTOMER=WILE_E_COYOTE; path=/; SameSite=test"); // invalid SameSite mode
+        Assert.assertEquals("CUSTOMER", cookie.getName());
+        Assert.assertEquals("WILE_E_COYOTE", cookie.getValue());
+        Assert.assertEquals("/", cookie.getPath());
+        Assert.assertNull(cookie.getSameSiteMode());
     }
 
     // RFC6265 allows US-ASCII characters excluding CTLs, whitespace,
