@@ -156,6 +156,7 @@ public final class HttpServletResponseImpl implements HttpServletResponse {
         writer = null;
         responseState = ResponseState.NONE;
         resetBuffer();
+        exchange.getResponseHeaders().remove(Headers.CONTENT_LENGTH);
         treatAsCommitted = false;
         final String location = servletContext.getDeployment().getErrorPages().getErrorLocation(sc);
         if (location != null) {
@@ -190,6 +191,7 @@ public final class HttpServletResponseImpl implements HttpServletResponse {
             throw UndertowServletMessages.MESSAGES.responseAlreadyCommited();
         }
         resetBuffer();
+        exchange.getResponseHeaders().remove(Headers.CONTENT_LENGTH);
         setStatus(StatusCodes.FOUND);
         String realPath;
         if (isAbsoluteUrl(location)) {//absolute url
@@ -203,6 +205,8 @@ public final class HttpServletResponseImpl implements HttpServletResponse {
                 int lastSlash = current.lastIndexOf("/");
                 if (lastSlash != -1) {
                     current = current.substring(0, lastSlash + 1);
+                } else {
+                    current = "";
                 }
                 realPath = CanonicalPathUtils.canonicalize(servletContext.getContextPath() + current + location);
             }
